@@ -16,10 +16,14 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+
 package org.apache.comet.shims
 
+import scala.annotation.nowarn
+
 import org.apache.spark.sql.types.DataType
 
 trait CometTypeShim {
-    def isStringCollationType(dt: DataType): Boolean = false
-}
\ No newline at end of file
+  @nowarn // Spark 4 feature; stubbed to false in Spark 3.x for compatibility.
+  def isStringCollationType(dt: DataType): Boolean = false
+}
