@@ -24,11 +24,10 @@ import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.execution.datasources.PartitionedFile
 
 object ShimBatchReader {
-
   def newPartitionedFile(partitionValues: InternalRow, file: String): PartitionedFile =
     PartitionedFile(
       partitionValues,
-      SparkPath.fromPathString(file),
+      SparkPath.fromUrlString(file),
       -1, // -1 means we read the entire file
       -1,
       Array.empty[String],
