@@ -25,5 +25,5 @@ import org.apache.spark.util.AccumulatorV2
 object ShimTaskMetrics {
 
   def getTaskAccumulator(taskMetrics: TaskMetrics): Option[AccumulatorV2[_, _]] =
-    taskMetrics.externalAccums.lastOption
+    taskMetrics._externalAccums.lastOption
 }
