@@ -41,11 +41,14 @@ use datafusion::{
 };
 use datafusion_comet_proto::spark_operator::Operator;
 use datafusion_spark::function::bitwise::bit_get::SparkBitGet;
+use datafusion_spark::function::bitwise::bitwise_not::SparkBitwiseNot;
 use datafusion_spark::function::datetime::date_add::SparkDateAdd;
 use datafusion_spark::function::datetime::date_sub::SparkDateSub;
+use datafusion_spark::function::datetime::last_day::SparkLastDay;
 use datafusion_spark::function::hash::sha1::SparkSha1;
 use datafusion_spark::function::hash::sha2::SparkSha2;
 use datafusion_spark::function::math::expm1::SparkExpm1;
+use datafusion_spark::function::math::hex::SparkHex;
 use datafusion_spark::function::string::char::CharFunc;
 use datafusion_spark::function::string::concat::SparkConcat;
 use futures::poll;
@@ -134,6 +137,8 @@ struct ExecutionContext {
     pub metrics_update_interval: Option<Duration>,
     // The last update time of metrics
     pub metrics_last_update_time: Instant,
+    /// Counter to avoid checking time on every poll iteration (reduces syscalls)
+    pub poll_count_since_metrics_check: u32,
     /// The time it took to create the native plan and configure the context
     pub plan_creation_time: Duration,
     /// DataFusion SessionContext
@@ -170,6 +175,7 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_createPlan(
     memory_limit: jlong,
     memory_limit_per_task: jlong,
     task_attempt_id: jlong,
+    task_cpus: jlong,
     key_unwrapper_obj: JObject,
 ) -> jlong {
     try_unwrap_or_throw(&e, |mut env| {
@@ -237,6 +243,7 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_createPlan(
                 memory_pool,
                 local_dirs_vec,
                 max_temp_directory_size,
+                task_cpus as usize,
             )?;
 
             let plan_creation_time = start.elapsed();
@@ -270,6 +277,7 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_createPlan(
                 metrics,
                 metrics_update_interval,
                 metrics_last_update_time: Instant::now(),
+                poll_count_since_metrics_check: 0,
                 plan_creation_time,
                 session_ctx: Arc::new(session),
                 debug_native,
@@ -289,6 +297,7 @@ fn prepare_datafusion_session_context(
     memory_pool: Arc<dyn MemoryPool>,
     local_dirs: Vec<String>,
     max_temp_directory_size: u64,
+    task_cpus: usize,
 ) -> CometResult<SessionContext> {
     let paths = local_dirs.into_iter().map(PathBuf::from).collect();
     let disk_manager = DiskManagerBuilder::default()
@@ -301,6 +310,10 @@ fn prepare_datafusion_session_context(
     // can be configured in Comet Spark JVM using Spark --conf parameters
     // e.g: spark-shell --conf spark.datafusion.sql_parser.parse_float_as_decimal=true
     let session_config = SessionConfig::new()
+        .with_target_partitions(task_cpus)
+        // This DataFusion context is within the scope of an executing Spark Task. We want to set
+        // its internal parallelism to the number of CPUs allocated to Spark Tasks. This can be
+        // modified by changing spark.task.cpus in the Spark config.
         .with_batch_size(batch_size)
         // DataFusion partial aggregates can emit duplicate rows so we disable the
         // skip partial aggregation feature because this is not compatible with Spark's
@@ -333,8 +346,11 @@ fn register_datafusion_spark_function(session_ctx: &SessionContext) {
     session_ctx.register_udf(ScalarUDF::new_from_impl(SparkBitGet::default()));
     session_ctx.register_udf(ScalarUDF::new_from_impl(SparkDateAdd::default()));
     session_ctx.register_udf(ScalarUDF::new_from_impl(SparkDateSub::default()));
+    session_ctx.register_udf(ScalarUDF::new_from_impl(SparkLastDay::default()));
     session_ctx.register_udf(ScalarUDF::new_from_impl(SparkSha1::default()));
     session_ctx.register_udf(ScalarUDF::new_from_impl(SparkConcat::default()));
+    session_ctx.register_udf(ScalarUDF::new_from_impl(SparkBitwiseNot::default()));
+    session_ctx.register_udf(ScalarUDF::new_from_impl(SparkHex::default()));
 }
 
 /// Prepares arrow arrays for output.
@@ -487,72 +503,88 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_executePlan(
                 let task_ctx = exec_context.session_ctx.task_ctx();
                 // Each Comet native execution corresponds to a single Spark partition,
                 // so we should always execute partition 0.
-                let stream = exec_context
-                    .root_op
-                    .as_ref()
-                    .unwrap()
-                    .native_plan
-                    .execute(0, task_ctx)?;
+                let stream = root_op.native_plan.execute(0, task_ctx)?;
                 exec_context.stream = Some(stream);
             } else {
                 // Pull input batches
                 pull_input_batches(exec_context)?;
             }
 
-            loop {
-                // Polling the stream.
-                let next_item = exec_context.stream.as_mut().unwrap().next();
-                let poll_output = get_runtime().block_on(async { poll!(next_item) });
+            // Enter the runtime once for the entire polling loop to avoid repeated
+            // Runtime::enter() overhead
+            get_runtime().block_on(async {
+                loop {
+                    // Polling the stream.
+                    let next_item = exec_context.stream.as_mut().unwrap().next();
+                    let poll_output = poll!(next_item);
 
-                // update metrics at interval
-                if let Some(interval) = exec_context.metrics_update_interval {
-                    let now = Instant::now();
-                    if now - exec_context.metrics_last_update_time >= interval {
-                        update_metrics(&mut env, exec_context)?;
-                        exec_context.metrics_last_update_time = now;
-                    }
-                }
-
-                match poll_output {
-                    Poll::Ready(Some(output)) => {
-                        // prepare output for FFI transfer
-                        return prepare_output(
-                            &mut env,
-                            array_addrs,
-                            schema_addrs,
-                            output?,
-                            exec_context.debug_native,
-                        );
-                    }
-                    Poll::Ready(None) => {
-                        // Reaches EOF of output.
-                        if exec_context.explain_native {
-                            if let Some(plan) = &exec_context.root_op {
-                                let formatted_plan_str = DisplayableExecutionPlan::with_metrics(
-                                    plan.native_plan.as_ref(),
-                                )
-                                .indent(true);
-                                info!(
-                                    "Comet native query plan with metrics (Plan #{} Stage {} Partition {}):\
-                                \n plan creation (including CometScans fetching first batches) took {:?}:\
-                                \n{formatted_plan_str:}",
-                                    plan.plan_id, stage_id, partition, exec_context.plan_creation_time
-                                );
+                    // update metrics at interval
+                    // Only check time every 100 polls to reduce syscall overhead
+                    if let Some(interval) = exec_context.metrics_update_interval {
+                        exec_context.poll_count_since_metrics_check += 1;
+                        if exec_context.poll_count_since_metrics_check >= 100 {
+                            let now = Instant::now();
+                            if now - exec_context.metrics_last_update_time >= interval {
+                                update_metrics(&mut env, exec_context)?;
+                                exec_context.metrics_last_update_time = now;
                             }
+                            exec_context.poll_count_since_metrics_check = 0;
                         }
-                        return Ok(-1);
                     }
-                    // A poll pending means there are more than one blocking operators,
-                    // we don't need go back-forth between JVM/Native. Just keeping polling.
-                    Poll::Pending => {
-                        // Pull input batches
-                        pull_input_batches(exec_context)?;
 
-                        // Output not ready yet
-                        continue;
+                    match poll_output {
+                        Poll::Ready(Some(output)) => {
+                            // prepare output for FFI transfer
+                            return prepare_output(
+                                &mut env,
+                                array_addrs,
+                                schema_addrs,
+                                output?,
+                                exec_context.debug_native,
+                            );
+                        }
+                        Poll::Ready(None) => {
+                            // Reaches EOF of output.
+                            if exec_context.explain_native {
+                                if let Some(plan) = &exec_context.root_op {
+                                    let formatted_plan_str = DisplayableExecutionPlan::with_metrics(
+                                        plan.native_plan.as_ref(),
+                                    )
+                                    .indent(true);
+                                    info!(
+                                        "Comet native query plan with metrics (Plan #{} Stage {} Partition {}):\
+                                    \n plan creation took {:?}:\
+                                    \n{formatted_plan_str:}",
+                                        plan.plan_id, stage_id, partition, exec_context.plan_creation_time
+                                    );
+                                }
+                            }
+                            return Ok(-1);
+                        }
+                        // A poll pending means the stream is not ready yet.
+                        Poll::Pending => {
+                            if exec_context.scans.is_empty() {
+                                // Pure async I/O (e.g., IcebergScanExec, DataSourceExec)
+                                // Yield to let the executor drive I/O instead of busy-polling
+                                tokio::task::yield_now().await;
+                            } else {
+                                // Has ScanExec operators
+                                // Busy-poll to pull batches from JVM
+                                // TODO: Investigate if JNI calls are safe without block_in_place.
+                                // block_in_place prevents Tokio from migrating this task to another thread,
+                                // which is necessary because JNI env is thread-local. If we can guarantee
+                                // thread safety another way, we could remove this wrapper for better perf.
+                                tokio::task::block_in_place(|| {
+                                    pull_input_batches(exec_context)
+                                })?;
+                            }
+
+                            // Output not ready yet
+                            continue;
+                        }
                     }
                 }
-            }
+            })
         })
     })
 }
@@ -582,8 +614,7 @@ pub extern "system" fn Java_org_apache_comet_Native_releasePlan(
 
 /// Updates the metrics of the query plan.
 fn update_metrics(env: &mut JNIEnv, exec_context: &mut ExecutionContext) -> CometResult<()> {
-    if exec_context.root_op.is_some() {
-        let native_query = exec_context.root_op.as_ref().unwrap();
+    if let Some(native_query) = &exec_context.root_op {
         let metrics = exec_context.metrics.as_obj();
         update_comet_metric(env, metrics, native_query)
     } else {
@@ -797,3 +828,121 @@ pub unsafe extern "system" fn Java_org_apache_comet_Native_logMemoryUsage(
         Ok(())
     })
 }
+
+// ============================================================================
+// Native Columnar to Row Conversion
+// ============================================================================
+
+use crate::execution::columnar_to_row::ColumnarToRowContext;
+use arrow::ffi::{from_ffi, FFI_ArrowArray, FFI_ArrowSchema};
+
+/// Initialize a native columnar to row converter.
+///
+/// # Safety
+/// This function is inherently unsafe since it deals with raw pointers passed from JNI.
+#[no_mangle]
+pub unsafe extern "system" fn Java_org_apache_comet_Native_columnarToRowInit(
+    e: JNIEnv,
+    _class: JClass,
+    serialized_schema: JObjectArray,
+    batch_size: jint,
+) -> jlong {
+    try_unwrap_or_throw(&e, |mut env| {
+        // Deserialize the schema
+        let schema = convert_datatype_arrays(&mut env, serialized_schema)?;
+
+        // Create the context
+        let ctx = Box::new(ColumnarToRowContext::new(schema, batch_size as usize));
+
+        Ok(Box::into_raw(ctx) as jlong)
+    })
+}
+
+/// Convert Arrow columnar data to Spark UnsafeRow format.
+///
+/// # Safety
+/// This function is inherently unsafe since it deals with raw pointers passed from JNI.
+#[no_mangle]
+pub unsafe extern "system" fn Java_org_apache_comet_Native_columnarToRowConvert(
+    e: JNIEnv,
+    _class: JClass,
+    c2r_handle: jlong,
+    array_addrs: JLongArray,
+    schema_addrs: JLongArray,
+    num_rows: jint,
+) -> jni::sys::jobject {
+    try_unwrap_or_throw(&e, |mut env| {
+        // Get the context
+        let ctx = (c2r_handle as *mut ColumnarToRowContext)
+            .as_mut()
+            .ok_or_else(|| CometError::Internal("Null columnar to row context".to_string()))?;
+
+        let num_cols = env.get_array_length(&array_addrs)? as usize;
+
+        // Get array and schema addresses
+        let array_addrs_elements = env.get_array_elements(&array_addrs, ReleaseMode::NoCopyBack)?;
+        let schema_addrs_elements =
+            env.get_array_elements(&schema_addrs, ReleaseMode::NoCopyBack)?;
+
+        // Import Arrow arrays from FFI
+        let mut arrays = Vec::with_capacity(num_cols);
+        for i in 0..num_cols {
+            let array_ptr = array_addrs_elements[i] as *mut FFI_ArrowArray;
+            let schema_ptr = schema_addrs_elements[i] as *mut FFI_ArrowSchema;
+
+            // Take ownership of the FFI structures
+            let ffi_array = std::ptr::read(array_ptr);
+            let ffi_schema = std::ptr::read(schema_ptr);
+
+            // Convert to Arrow ArrayData
+            let array_data = from_ffi(ffi_array, &ffi_schema)
+                .map_err(|e| CometError::Internal(format!("Failed to import array: {}", e)))?;
+
+            arrays.push(arrow::array::make_array(array_data));
+        }
+
+        // Convert columnar to row
+        let (buffer_ptr, offsets, lengths) = ctx.convert(&arrays, num_rows as usize)?;
+
+        // Create Java int arrays for offsets and lengths
+        let offsets_array = env.new_int_array(offsets.len() as i32)?;
+        env.set_int_array_region(&offsets_array, 0, offsets)?;
+
+        let lengths_array = env.new_int_array(lengths.len() as i32)?;
+        env.set_int_array_region(&lengths_array, 0, lengths)?;
+
+        // Create the NativeColumnarToRowInfo object
+        let info_class = env.find_class("org/apache/comet/NativeColumnarToRowInfo")?;
+        let info_obj = env.new_object(
+            info_class,
+            "(J[I[I)V",
+            &[
+                jni::objects::JValue::Long(buffer_ptr as jlong),
+                jni::objects::JValue::Object(&offsets_array),
+                jni::objects::JValue::Object(&lengths_array),
+            ],
+        )?;
+
+        Ok(info_obj.into_raw())
+    })
+}
+
+/// Close and release the native columnar to row converter.
+///
+/// # Safety
+/// This function is inherently unsafe since it deals with raw pointers passed from JNI.
+#[no_mangle]
+pub unsafe extern "system" fn Java_org_apache_comet_Native_columnarToRowClose(
+    e: JNIEnv,
+    _class: JClass,
+    c2r_handle: jlong,
+) {
+    try_unwrap_or_throw(&e, |_env| {
+        if c2r_handle != 0 {
+            let _ctx: Box<ColumnarToRowContext> =
+                Box::from_raw(c2r_handle as *mut ColumnarToRowContext);
+            // ctx is dropped here, freeing the buffer
+        }
+        Ok(())
+    })
+}
