@@ -15,10 +15,9 @@
 // specific language governing permissions and limitations
 // under the License.
 
-pub mod file_util;
 pub mod page_util;
 pub mod rand_gen;
 
 pub use self::rand_gen::{random_bools, random_bytes, random_numbers, random_numbers_range};
 
-pub use self::file_util::{get_temp_file, get_temp_filename};
+pub use datafusion_comet_spark_expr::test_common::file_util::{get_temp_file, get_temp_filename};
