@@ -19,8 +19,10 @@ mod array_insert;
 mod array_repeat;
 mod get_array_struct_fields;
 mod list_extract;
+mod size;
 
 pub use array_insert::ArrayInsert;
 pub use array_repeat::spark_array_repeat;
 pub use get_array_struct_fields::GetArrayStructFields;
 pub use list_extract::ListExtract;
+pub use size::{spark_size, SparkSizeFunc};
