@@ -16,7 +16,5 @@
 // under the License.
 
 mod bitwise_count;
-mod bitwise_not;
 
 pub use bitwise_count::SparkBitwiseCount;
-pub use bitwise_not::SparkBitwiseNot;
