@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+mod from_json;
 mod to_json;
 
+pub use from_json::FromJson;
 pub use to_json::ToJson;
