@@ -20,7 +20,6 @@ mod ceil;
 pub(crate) mod checked_arithmetic;
 mod div;
 mod floor;
-pub(crate) mod hex;
 pub mod internal;
 pub mod modulo_expr;
 mod negative;
@@ -32,7 +31,6 @@ pub use ceil::spark_ceil;
 pub use div::spark_decimal_div;
 pub use div::spark_decimal_integral_div;
 pub use floor::spark_floor;
-pub use hex::spark_hex;
 pub use internal::*;
 pub use modulo_expr::create_modulo_expr;
 pub use negative::{create_negate_expr, NegativeExpr};
