@@ -19,8 +19,12 @@
 
 package org.apache.comet.serde.operator
 
+import scala.collection.mutable
 import scala.jdk.CollectionConverters._
 
+import org.json4s.JsonDSL._
+import org.json4s.jackson.JsonMethods._
+
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.catalyst.expressions._
 import org.apache.spark.sql.comet.{CometBatchScanExec, CometNativeExec}
@@ -29,6 +33,7 @@ import org.apache.spark.sql.types._
 import org.apache.comet.ConfigEntry
 import org.apache.comet.iceberg.IcebergReflection
 import org.apache.comet.serde.{CometOperatorSerde, OperatorOuterClass}
+import org.apache.comet.serde.ExprOuterClass.Expr
 import org.apache.comet.serde.OperatorOuterClass.{Operator, SparkStructField}
 import org.apache.comet.serde.QueryPlanSerde.{exprToProto, serializeDataType}
 
@@ -64,6 +69,136 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
     }
   }
 
+  /**
+   * Converts an Iceberg partition value to protobuf format. Protobuf is less verbose than JSON.
+   * The following types are also serialized as integer values instead of as strings - Timestamps,
+   * Dates, Decimals, FieldIDs
+   */
+  private def partitionValueToProto(
+      fieldId: Int,
+      fieldTypeStr: String,
+      value: Any): OperatorOuterClass.PartitionValue = {
+    val builder = OperatorOuterClass.PartitionValue.newBuilder()
+    builder.setFieldId(fieldId)
+
+    if (value == null) {
+      builder.setIsNull(true)
+    } else {
+      builder.setIsNull(false)
+      fieldTypeStr match {
+        case t if t.startsWith("timestamp") =>
+          val micros = value match {
+            case l: java.lang.Long => l.longValue()
+            case i: java.lang.Integer => i.longValue()
+            case _ => value.toString.toLong
+          }
+          if (t.contains("tz")) {
+            builder.setTimestampTzVal(micros)
+          } else {
+            builder.setTimestampVal(micros)
+          }
+
+        case "date" =>
+          val days = value.asInstanceOf[java.lang.Integer].intValue()
+          builder.setDateVal(days)
+
+        case d if d.startsWith("decimal(") =>
+          // Serialize as unscaled BigInteger bytes
+          val bigDecimal = value match {
+            case bd: java.math.BigDecimal => bd
+            case _ => new java.math.BigDecimal(value.toString)
+          }
+          val unscaledBytes = bigDecimal.unscaledValue().toByteArray
+          builder.setDecimalVal(com.google.protobuf.ByteString.copyFrom(unscaledBytes))
+
+        case "string" =>
+          builder.setStringVal(value.toString)
+
+        case "int" =>
+          val intVal = value match {
+            case i: java.lang.Integer => i.intValue()
+            case l: java.lang.Long => l.intValue()
+            case _ => value.toString.toInt
+          }
+          builder.setIntVal(intVal)
+
+        case "long" =>
+          val longVal = value match {
+            case l: java.lang.Long => l.longValue()
+            case i: java.lang.Integer => i.longValue()
+            case _ => value.toString.toLong
+          }
+          builder.setLongVal(longVal)
+
+        case "float" =>
+          val floatVal = value match {
+            case f: java.lang.Float => f.floatValue()
+            case d: java.lang.Double => d.floatValue()
+            case _ => value.toString.toFloat
+          }
+          builder.setFloatVal(floatVal)
+
+        case "double" =>
+          val doubleVal = value match {
+            case d: java.lang.Double => d.doubleValue()
+            case f: java.lang.Float => f.doubleValue()
+            case _ => value.toString.toDouble
+          }
+          builder.setDoubleVal(doubleVal)
+
+        case "boolean" =>
+          val boolVal = value match {
+            case b: java.lang.Boolean => b.booleanValue()
+            case _ => value.toString.toBoolean
+          }
+          builder.setBoolVal(boolVal)
+
+        case "uuid" =>
+          // UUID as bytes (16 bytes) or string
+          val uuidBytes = value match {
+            case uuid: java.util.UUID =>
+              val bb = java.nio.ByteBuffer.wrap(new Array[Byte](16))
+              bb.putLong(uuid.getMostSignificantBits)
+              bb.putLong(uuid.getLeastSignificantBits)
+              bb.array()
+            case _ =>
+              // Parse UUID string and convert to bytes
+              val uuid = java.util.UUID.fromString(value.toString)
+              val bb = java.nio.ByteBuffer.wrap(new Array[Byte](16))
+              bb.putLong(uuid.getMostSignificantBits)
+              bb.putLong(uuid.getLeastSignificantBits)
+              bb.array()
+          }
+          builder.setUuidVal(com.google.protobuf.ByteString.copyFrom(uuidBytes))
+
+        case t if t.startsWith("fixed[") || t.startsWith("binary") =>
+          val bytes = value match {
+            case bytes: Array[Byte] => bytes
+            case _ => value.toString.getBytes("UTF-8")
+          }
+          if (t.startsWith("fixed")) {
+            builder.setFixedVal(com.google.protobuf.ByteString.copyFrom(bytes))
+          } else {
+            builder.setBinaryVal(com.google.protobuf.ByteString.copyFrom(bytes))
+          }
+
+        // Fallback: infer type from Java type ?
+        case _ =>
+          value match {
+            case s: String => builder.setStringVal(s)
+            case i: java.lang.Integer => builder.setIntVal(i.intValue())
+            case l: java.lang.Long => builder.setLongVal(l.longValue())
+            case d: java.lang.Double => builder.setDoubleVal(d.doubleValue())
+            case f: java.lang.Float => builder.setFloatVal(f.floatValue())
+            case b: java.lang.Boolean => builder.setBoolVal(b.booleanValue())
+            case other => builder.setStringVal(other.toString)
+          }
+      }
+    }
+
+    builder.build()
+  }
+
   /**
    * Helper to extract a literal from an Iceberg expression and build a binary predicate.
    */
@@ -83,17 +218,12 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
   }
 
   /**
-   * Serializes delete files from an Iceberg FileScanTask.
-   *
-   * Extracts delete files (position deletes and equality deletes) from a FileScanTask and adds
-   * them to the task builder. Delete files are used for Iceberg's merge-on-read approach where
-   * updates and deletes are stored separately from data files.
+   * Extracts delete files from an Iceberg FileScanTask as a list (for deduplication).
    */
-  private def serializeDeleteFiles(
+  private def extractDeleteFilesList(
       task: Any,
       contentFileClass: Class[_],
-      fileScanTaskClass: Class[_],
-      taskBuilder: OperatorOuterClass.IcebergFileScanTask.Builder): Unit = {
+      fileScanTaskClass: Class[_]): Seq[OperatorOuterClass.IcebergDeleteFile] = {
     try {
       // scalastyle:off classforname
       val deleteFileClass = Class.forName(IcebergReflection.ClassNames.DELETE_FILE)
@@ -101,11 +231,11 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
 
       val deletes = IcebergReflection.getDeleteFilesFromTask(task, fileScanTaskClass)
 
-      deletes.asScala.foreach { deleteFile =>
+      deletes.asScala.flatMap { deleteFile =>
         try {
           IcebergReflection
             .extractFileLocation(contentFileClass, deleteFile)
-            .foreach { deletePath =>
+            .map { deletePath =>
               val deleteBuilder =
                 OperatorOuterClass.IcebergDeleteFile.newBuilder()
               deleteBuilder.setFilePath(deletePath)
@@ -148,13 +278,14 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                 case _: Exception =>
               }
 
-              taskBuilder.addDeleteFiles(deleteBuilder.build())
+              deleteBuilder.build()
             }
         } catch {
           case e: Exception =>
             logWarning(s"Failed to serialize delete file: ${e.getMessage}")
+            None
         }
-      }
+      }.toSeq
     } catch {
       case e: Exception =>
         val msg =
@@ -177,13 +308,17 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
       task: Any,
       contentScanTaskClass: Class[_],
       fileScanTaskClass: Class[_],
-      taskBuilder: OperatorOuterClass.IcebergFileScanTask.Builder): Unit = {
+      taskBuilder: OperatorOuterClass.IcebergFileScanTask.Builder,
+      icebergScanBuilder: OperatorOuterClass.IcebergScan.Builder,
+      partitionTypeToPoolIndex: mutable.HashMap[String, Int],
+      partitionSpecToPoolIndex: mutable.HashMap[String, Int],
+      partitionDataToPoolIndex: mutable.HashMap[String, Int]): Unit = {
     try {
       val specMethod = fileScanTaskClass.getMethod("spec")
       val spec = specMethod.invoke(task)
 
       if (spec != null) {
-        // Serialize the entire PartitionSpec to JSON (includes spec-id)
+        // Deduplicate partition spec
         try {
           // scalastyle:off classforname
           val partitionSpecParserClass =
@@ -195,7 +330,14 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
           val partitionSpecJson = toJsonMethod
             .invoke(null, spec)
             .asInstanceOf[String]
-          taskBuilder.setPartitionSpecJson(partitionSpecJson)
+
+          val specIdx = partitionSpecToPoolIndex.getOrElseUpdate(
+            partitionSpecJson, {
+              val idx = partitionSpecToPoolIndex.size
+              icebergScanBuilder.addPartitionSpecPool(partitionSpecJson)
+              idx
+            })
+          taskBuilder.setPartitionSpecIdx(specIdx)
         } catch {
           case e: Exception =>
             logWarning(s"Failed to serialize partition spec to JSON: ${e.getMessage}")
@@ -230,8 +372,6 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
               // metadata (e.g., "schema-id") that iceberg-rust's StructType
               // deserializer rejects. We need pure StructType format:
               // {"type":"struct","fields":[...]}
-              import org.json4s.JsonDSL._
-              import org.json4s.jackson.JsonMethods._
 
               // Filter out fields with unknown types (dropped partition fields).
               // Unknown type fields represent partition columns that have been dropped
@@ -272,7 +412,13 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                     ("type" -> "struct") ~
                       ("fields" -> fieldsJson)))
 
-                taskBuilder.setPartitionTypeJson(partitionTypeJson)
+                val typeIdx = partitionTypeToPoolIndex.getOrElseUpdate(
+                  partitionTypeJson, {
+                    val idx = partitionTypeToPoolIndex.size
+                    icebergScanBuilder.addPartitionTypePool(partitionTypeJson)
+                    idx
+                  })
+                taskBuilder.setPartitionTypeIdx(typeIdx)
               }
             } catch {
               case e: Exception =>
@@ -280,17 +426,17 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
             }
           }
 
-          // Serialize partition data to JSON for iceberg-rust's constants_map.
-          // The native execution engine uses partition_data_json +
-          // partition_type_json to build a constants_map, which is the primary
-          // mechanism for providing partition values to identity-transformed
-          // partition columns. Non-identity transforms (bucket, truncate, days,
-          // etc.) read values from data files.
-          import org.json4s._
-          import org.json4s.jackson.JsonMethods._
+          // Serialize partition data to protobuf for native execution.
+          // The native execution engine uses partition_data protobuf messages to
+          // build a constants_map, which provides partition values to identity-
+          // transformed partition columns. Non-identity transforms (bucket, truncate,
+          // days, etc.) read values from data files.
+          //
+          // IMPORTANT: Use partition field IDs (not source field IDs) to match
+          // the schema.
 
           // Filter out fields with unknown type (same as partition type filtering)
-          val partitionDataMap: Map[String, JValue] =
+          val partitionValues: Seq[OperatorOuterClass.PartitionValue] =
             fields.asScala.zipWithIndex.flatMap { case (field, idx) =>
               val fieldTypeStr = getFieldType(field)
 
@@ -298,6 +444,7 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
               if (fieldTypeStr == IcebergReflection.TypeNames.UNKNOWN) {
                 None
               } else {
+                // Use the partition type's field ID (same as in partition_type_json)
                 val fieldIdMethod = field.getClass.getMethod("fieldId")
                 val fieldId = fieldIdMethod.invoke(field).asInstanceOf[Int]
 
@@ -305,34 +452,28 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                   partitionData.getClass.getMethod("get", classOf[Int], classOf[Class[_]])
                 val value = getMethod.invoke(partitionData, Integer.valueOf(idx), classOf[Object])
 
-                val jsonValue: JValue = if (value == null) {
-                  JNull
-                } else {
-                  value match {
-                    case s: String => JString(s)
-                    // NaN/Infinity are not valid JSON number literals per the
-                    // JSON spec. Serialize as strings (e.g., "NaN", "Infinity")
-                    // which are valid JSON and can be parsed by Rust's
-                    // f32/f64::from_str().
-                    case f: java.lang.Float if f.isNaN || f.isInfinite =>
-                      JString(f.toString)
-                    case d: java.lang.Double if d.isNaN || d.isInfinite =>
-                      JString(d.toString)
-                    case n: Number => JDecimal(BigDecimal(n.toString))
-                    case b: java.lang.Boolean =>
-                      JBool(b.booleanValue())
-                    case other => JString(other.toString)
-                  }
-                }
-
-                Some(fieldId.toString -> jsonValue)
+                Some(partitionValueToProto(fieldId, fieldTypeStr, value))
               }
-            }.toMap
+            }.toSeq
 
           // Only serialize partition data if we have non-unknown fields
-          if (partitionDataMap.nonEmpty) {
-            val partitionJson = compact(render(JObject(partitionDataMap.toList)))
-            taskBuilder.setPartitionDataJson(partitionJson)
+          if (partitionValues.nonEmpty) {
+            val partitionDataProto = OperatorOuterClass.PartitionData
+              .newBuilder()
+              .addAllValues(partitionValues.asJava)
+              .build()
+
+            // Deduplicate by protobuf bytes (use Base64 string as key)
+            val partitionDataBytes = partitionDataProto.toByteArray
+            val partitionDataKey = java.util.Base64.getEncoder.encodeToString(partitionDataBytes)
+
+            val partitionDataIdx = partitionDataToPoolIndex.getOrElseUpdate(
+              partitionDataKey, {
+                val idx = partitionDataToPoolIndex.size
+                icebergScanBuilder.addPartitionDataPool(partitionDataProto)
+                idx
+              })
+            taskBuilder.setPartitionDataIdx(partitionDataIdx)
           }
         }
       }
@@ -359,6 +500,7 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
         // Global S3A configuration keys
         case "fs.s3a.access.key" => Some("s3.access-key-id" -> value)
         case "fs.s3a.secret.key" => Some("s3.secret-access-key" -> value)
+        case "fs.s3a.session.token" => Some("s3.session-token" -> value)
         case "fs.s3a.endpoint" => Some("s3.endpoint" -> value)
         case "fs.s3a.path.style.access" => Some("s3.path-style-access" -> value)
         case "fs.s3a.endpoint.region" => Some("s3.region" -> value)
@@ -373,6 +515,7 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
             property match {
               case "access.key" => Some(s"s3.bucket.$bucket.access-key-id" -> value)
               case "secret.key" => Some(s"s3.bucket.$bucket.secret-access-key" -> value)
+              case "session.token" => Some(s"s3.bucket.$bucket.session.token" -> value)
               case "endpoint" => Some(s"s3.bucket.$bucket.endpoint" -> value)
               case "path.style.access" => Some(s"s3.bucket.$bucket.path-style-access" -> value)
               case "endpoint.region" => Some(s"s3.bucket.$bucket.region" -> value)
@@ -540,6 +683,19 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
       childOp: Operator*): Option[OperatorOuterClass.Operator] = {
     val icebergScanBuilder = OperatorOuterClass.IcebergScan.newBuilder()
 
+    // Deduplication structures - map unique values to pool indices
+    val schemaToPoolIndex = mutable.HashMap[AnyRef, Int]()
+    val partitionTypeToPoolIndex = mutable.HashMap[String, Int]()
+    val partitionSpecToPoolIndex = mutable.HashMap[String, Int]()
+    val nameMappingToPoolIndex = mutable.HashMap[String, Int]()
+    val projectFieldIdsToPoolIndex = mutable.HashMap[Seq[Int], Int]()
+    val partitionDataToPoolIndex = mutable.HashMap[String, Int]() // Base64 bytes -> pool index
+    val deleteFilesToPoolIndex =
+      mutable.HashMap[Seq[OperatorOuterClass.IcebergDeleteFile], Int]()
+    val residualToPoolIndex = mutable.HashMap[Option[Expr], Int]()
+
+    var totalTasks = 0
+
     // Get pre-extracted metadata from planning phase
     // If metadata is None, this is a programming error - metadata should have been extracted
     // in CometScanRule before creating CometBatchScanExec
@@ -593,6 +749,8 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                   tasksMethod.invoke(taskGroup).asInstanceOf[java.util.Collection[_]]
 
                 tasksCollection.asScala.foreach { task =>
+                  totalTasks += 1
+
                   try {
                     val taskBuilder = OperatorOuterClass.IcebergFileScanTask.newBuilder()
 
@@ -621,98 +779,6 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                         throw new RuntimeException(msg)
                     }
 
-                    // Extract partition values for Hive-style partitioning
-                    var partitionJsonOpt: Option[String] = None
-                    try {
-                      val partitionMethod = contentFileClass.getMethod("partition")
-                      val partitionStruct = partitionMethod.invoke(dataFile)
-
-                      if (partitionStruct != null) {
-                        // scalastyle:off classforname
-                        val structLikeClass =
-                          Class.forName(IcebergReflection.ClassNames.STRUCT_LIKE)
-                        // scalastyle:on classforname
-                        val sizeMethod = structLikeClass.getMethod("size")
-                        val getMethod =
-                          structLikeClass.getMethod("get", classOf[Int], classOf[Class[_]])
-
-                        val partitionSize =
-                          sizeMethod.invoke(partitionStruct).asInstanceOf[Int]
-
-                        if (partitionSize > 0) {
-                          // Get the partition spec directly from the task
-                          // scalastyle:off classforname
-                          val partitionScanTaskClass =
-                            Class.forName(IcebergReflection.ClassNames.PARTITION_SCAN_TASK)
-                          // scalastyle:on classforname
-                          val specMethod = partitionScanTaskClass.getMethod("spec")
-                          val partitionSpec = specMethod.invoke(task)
-
-                          // Build JSON representation of partition values using json4s
-                          import org.json4s._
-                          import org.json4s.jackson.JsonMethods._
-
-                          val partitionMap = scala.collection.mutable.Map[String, JValue]()
-
-                          if (partitionSpec != null) {
-                            // Get the list of partition fields from the spec
-                            val fieldsMethod = partitionSpec.getClass.getMethod("fields")
-                            val fields = fieldsMethod
-                              .invoke(partitionSpec)
-                              .asInstanceOf[java.util.List[_]]
-
-                            for (i <- 0 until partitionSize) {
-                              val value =
-                                getMethod.invoke(partitionStruct, Int.box(i), classOf[Object])
-
-                              // Get the partition field and check its transform type
-                              val partitionField = fields.get(i)
-
-                              // Only inject partition values for IDENTITY transforms
-                              val transformMethod =
-                                partitionField.getClass.getMethod("transform")
-                              val transform = transformMethod.invoke(partitionField)
-                              val isIdentity =
-                                transform.toString == IcebergReflection.Transforms.IDENTITY
-
-                              if (isIdentity) {
-                                // Get the source field ID
-                                val sourceIdMethod =
-                                  partitionField.getClass.getMethod("sourceId")
-                                val sourceFieldId =
-                                  sourceIdMethod.invoke(partitionField).asInstanceOf[Int]
-
-                                // Convert value to appropriate JValue type
-                                val jsonValue: JValue = if (value == null) {
-                                  JNull
-                                } else {
-                                  value match {
-                                    case s: String => JString(s)
-                                    case i: java.lang.Integer => JInt(BigInt(i.intValue()))
-                                    case l: java.lang.Long => JInt(BigInt(l.longValue()))
-                                    case d: java.lang.Double => JDouble(d.doubleValue())
-                                    case f: java.lang.Float => JDouble(f.doubleValue())
-                                    case b: java.lang.Boolean => JBool(b.booleanValue())
-                                    case n: Number => JDecimal(BigDecimal(n.toString))
-                                    case other => JString(other.toString)
-                                  }
-                                }
-
-                                partitionMap(sourceFieldId.toString) = jsonValue
-                              }
-                            }
-                          }
-
-                          val partitionJson = compact(render(JObject(partitionMap.toList)))
-                          partitionJsonOpt = Some(partitionJson)
-                        }
-                      }
-                    } catch {
-                      case e: Exception =>
-                        logWarning(
-                          s"Failed to extract partition values from DataFile: ${e.getMessage}")
-                    }
-
                     val startMethod = contentScanTaskClass.getMethod("start")
                     val start = startMethod.invoke(task).asInstanceOf[Long]
                     taskBuilder.setStart(start)
@@ -734,12 +800,47 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                         IcebergReflection.getDeleteFilesFromTask(task, fileScanTaskClass)
                       val hasDeletes = !deletes.isEmpty
 
-                      // Use pre-extracted scanSchema for schema evolution support
+                      // Schema to pass to iceberg-rust's FileScanTask.
+                      // This is used by RecordBatchTransformer for field type lookups (e.g., in
+                      // constants_map) and default value generation. The actual projection is
+                      // controlled by project_field_ids.
+                      //
+                      // Schema selection logic:
+                      // 1. If hasDeletes=true: Use taskSchema (file-specific schema) because
+                      // delete files reference specific schema versions and we need exact schema
+                      // matching for MOR.
+                      // 2. Else if scanSchema contains columns not in tableSchema: Use scanSchema
+                      // because this is a VERSION AS OF query reading a historical snapshot with
+                      // different schema (e.g., after column drop, scanSchema has old columns
+                      // that tableSchema doesn't)
+                      // 3. Else: Use tableSchema because scanSchema is the query OUTPUT schema
+                      // (e.g., for aggregates like "SELECT count(*)", scanSchema only has
+                      // aggregate fields and doesn't contain partition columns needed by
+                      // constants_map)
                       val schema: AnyRef =
                         if (hasDeletes) {
                           taskSchema
                         } else {
-                          metadata.scanSchema.asInstanceOf[AnyRef]
+                          // Check if scanSchema has columns that tableSchema doesn't have
+                          // (VERSION AS OF case)
+                          val scanSchemaFieldIds = IcebergReflection
+                            .buildFieldIdMapping(metadata.scanSchema)
+                            .values
+                            .toSet
+                          val tableSchemaFieldIds = IcebergReflection
+                            .buildFieldIdMapping(metadata.tableSchema)
+                            .values
+                            .toSet
+                          val hasHistoricalColumns =
+                            scanSchemaFieldIds.exists(id => !tableSchemaFieldIds.contains(id))
+
+                          if (hasHistoricalColumns) {
+                            // VERSION AS OF: scanSchema has columns that current table doesn't have
+                            metadata.scanSchema.asInstanceOf[AnyRef]
+                          } else {
+                            // Regular query: use tableSchema for partition field lookups
+                            metadata.tableSchema.asInstanceOf[AnyRef]
+                          }
                         }
 
                       // scalastyle:off classforname
@@ -749,9 +850,17 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                       // scalastyle:on classforname
                       val toJsonMethod = schemaParserClass.getMethod("toJson", schemaClass)
                       toJsonMethod.setAccessible(true)
-                      val schemaJson = toJsonMethod.invoke(null, schema).asInstanceOf[String]
 
-                      taskBuilder.setSchemaJson(schemaJson)
+                      // Use object identity for deduplication: Iceberg Schema objects are immutable
+                      // and reused across tasks, making identity-based deduplication safe
+                      val schemaIdx = schemaToPoolIndex.getOrElseUpdate(
+                        schema, {
+                          val idx = schemaToPoolIndex.size
+                          val schemaJson = toJsonMethod.invoke(null, schema).asInstanceOf[String]
+                          icebergScanBuilder.addSchemaPool(schemaJson)
+                          idx
+                        })
+                      taskBuilder.setSchemaIdx(schemaIdx)
 
                       // Build field ID mapping from the schema we're using
                       val nameToFieldId = IcebergReflection.buildFieldIdMapping(schema)
@@ -759,20 +868,28 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                       // Extract project_field_ids for scan.output columns.
                       // For schema evolution: try task schema first, then fall back to
                       // global scan schema (pre-extracted in metadata).
-                      scan.output.foreach { attr =>
-                        val fieldId = nameToFieldId
+                      val projectFieldIds = scan.output.flatMap { attr =>
+                        nameToFieldId
                           .get(attr.name)
                           .orElse(metadata.globalFieldIdMapping.get(attr.name))
-
-                        fieldId match {
-                          case Some(id) =>
-                            taskBuilder.addProjectFieldIds(id)
-                          case None =>
+                          .orElse {
                             logWarning(
                               s"Column '${attr.name}' not found in task or scan schema," +
                                 "skipping projection")
-                        }
+                            None
+                          }
                       }
+
+                      // Deduplicate project field IDs
+                      val projectFieldIdsIdx = projectFieldIdsToPoolIndex.getOrElseUpdate(
+                        projectFieldIds, {
+                          val idx = projectFieldIdsToPoolIndex.size
+                          val listBuilder = OperatorOuterClass.ProjectFieldIdList.newBuilder()
+                          projectFieldIds.foreach(id => listBuilder.addFieldIds(id))
+                          icebergScanBuilder.addProjectFieldIdsPool(listBuilder.build())
+                          idx
+                        })
+                      taskBuilder.setProjectFieldIdsIdx(projectFieldIdsIdx)
                     } catch {
                       case e: Exception =>
                         val msg =
@@ -783,29 +900,48 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                         throw new RuntimeException(msg, e)
                     }
 
-                    taskBuilder.setDataFileFormat(metadata.fileFormat)
+                    // Deduplicate delete files
+                    val deleteFilesList =
+                      extractDeleteFilesList(task, contentFileClass, fileScanTaskClass)
+                    if (deleteFilesList.nonEmpty) {
+                      val deleteFilesIdx = deleteFilesToPoolIndex.getOrElseUpdate(
+                        deleteFilesList, {
+                          val idx = deleteFilesToPoolIndex.size
+                          val listBuilder = OperatorOuterClass.DeleteFileList.newBuilder()
+                          deleteFilesList.foreach(df => listBuilder.addDeleteFiles(df))
+                          icebergScanBuilder.addDeleteFilesPool(listBuilder.build())
+                          idx
+                        })
+                      taskBuilder.setDeleteFilesIdx(deleteFilesIdx)
+                    }
 
-                    // Serialize delete files (position deletes and equality deletes)
-                    serializeDeleteFiles(task, contentFileClass, fileScanTaskClass, taskBuilder)
+                    // Extract and deduplicate residual expression
+                    val residualExprOpt =
+                      try {
+                        val residualMethod = contentScanTaskClass.getMethod("residual")
+                        val residualExpr = residualMethod.invoke(task)
 
-                    try {
-                      val residualMethod = contentScanTaskClass.getMethod("residual")
-                      val residualExpr = residualMethod.invoke(task)
+                        val catalystExpr = convertIcebergExpression(residualExpr, scan.output)
 
-                      val catalystExpr = convertIcebergExpression(residualExpr, scan.output)
-
-                      catalystExpr
-                        .flatMap { expr =>
+                        catalystExpr.flatMap { expr =>
                           exprToProto(expr, scan.output, binding = false)
                         }
-                        .foreach { protoExpr =>
-                          taskBuilder.setResidual(protoExpr)
-                        }
-                    } catch {
-                      case e: Exception =>
-                        logWarning(
-                          "Failed to extract residual expression from FileScanTask: " +
-                            s"${e.getMessage}")
+                      } catch {
+                        case e: Exception =>
+                          logWarning(
+                            "Failed to extract residual expression from FileScanTask: " +
+                              s"${e.getMessage}")
+                          None
+                      }
+
+                    residualExprOpt.foreach { residualExpr =>
+                      val residualIdx = residualToPoolIndex.getOrElseUpdate(
+                        Some(residualExpr), {
+                          val idx = residualToPoolIndex.size
+                          icebergScanBuilder.addResidualPool(residualExpr)
+                          idx
+                        })
+                      taskBuilder.setResidualIdx(residualIdx)
                     }
 
                     // Serialize partition spec and data (field definitions, transforms, values)
@@ -813,10 +949,22 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
                       task,
                       contentScanTaskClass,
                       fileScanTaskClass,
-                      taskBuilder)
+                      taskBuilder,
+                      icebergScanBuilder,
+                      partitionTypeToPoolIndex,
+                      partitionSpecToPoolIndex,
+                      partitionDataToPoolIndex)
 
-                    // Set name mapping if available (shared by all tasks, pre-extracted)
-                    metadata.nameMapping.foreach(taskBuilder.setNameMappingJson)
+                    // Deduplicate name mapping
+                    metadata.nameMapping.foreach { nm =>
+                      val nmIdx = nameMappingToPoolIndex.getOrElseUpdate(
+                        nm, {
+                          val idx = nameMappingToPoolIndex.size
+                          icebergScanBuilder.addNameMappingPool(nm)
+                          idx
+                        })
+                      taskBuilder.setNameMappingIdx(nmIdx)
+                    }
 
                     partitionBuilder.addFileScanTasks(taskBuilder.build())
                   }
@@ -838,6 +986,42 @@ object CometIcebergNativeScan extends CometOperatorSerde[CometBatchScanExec] wit
         return None
     }
 
+    // Log deduplication summary
+    val allPoolSizes = Seq(
+      schemaToPoolIndex.size,
+      partitionTypeToPoolIndex.size,
+      partitionSpecToPoolIndex.size,
+      nameMappingToPoolIndex.size,
+      projectFieldIdsToPoolIndex.size,
+      partitionDataToPoolIndex.size,
+      deleteFilesToPoolIndex.size,
+      residualToPoolIndex.size)
+
+    val avgDedup = if (totalTasks == 0) {
+      "0.0"
+    } else {
+      // Filter out empty pools - they shouldn't count as 100% dedup
+      val nonEmptyPools = allPoolSizes.filter(_ > 0)
+      if (nonEmptyPools.isEmpty) {
+        "0.0"
+      } else {
+        val avgUnique = nonEmptyPools.sum.toDouble / nonEmptyPools.length
+        f"${(1.0 - avgUnique / totalTasks) * 100}%.1f"
+      }
+    }
+
+    // Calculate partition data pool size in bytes (protobuf format)
+    val partitionDataPoolBytes = icebergScanBuilder.getPartitionDataPoolList.asScala
+      .map(_.getSerializedSize)
+      .sum
+
+    logInfo(s"IcebergScan: $totalTasks tasks, ${allPoolSizes.size} pools ($avgDedup% avg dedup)")
+    if (partitionDataToPoolIndex.nonEmpty) {
+      logInfo(
+        s"  Partition data pool: ${partitionDataToPoolIndex.size} unique values, " +
+          s"$partitionDataPoolBytes bytes (protobuf)")
+    }
+
     builder.clearChildren()
     Some(builder.setIcebergScan(icebergScanBuilder).build())
   }
