@@ -21,11 +21,11 @@ package org.apache.comet.serde
 
 import scala.jdk.CollectionConverters._
 
-import org.apache.spark.sql.catalyst.expressions.{Attribute, CreateNamedStruct, GetArrayStructFields, GetStructField, StructsToJson}
+import org.apache.spark.sql.catalyst.expressions.{Attribute, CreateNamedStruct, GetArrayStructFields, GetStructField, JsonToStructs, StructsToJson}
 import org.apache.spark.sql.types.{ArrayType, DataType, DataTypes, MapType, StructType}
 
 import org.apache.comet.CometSparkSessionExtensions.withInfo
-import org.apache.comet.serde.QueryPlanSerde.exprToProtoInternal
+import org.apache.comet.serde.QueryPlanSerde.{exprToProtoInternal, serializeDataType}
 
 object CometCreateNamedStruct extends CometExpressionSerde[CreateNamedStruct] {
   override def convert(
@@ -111,26 +111,6 @@ object CometStructsToJson extends CometExpressionSerde[StructsToJson] {
       withInfo(expr, "StructsToJson with options is not supported")
       None
     } else {
-
-      def isSupportedType(dt: DataType): Boolean = {
-        dt match {
-          case StructType(fields) =>
-            fields.forall(f => isSupportedType(f.dataType))
-          case DataTypes.BooleanType | DataTypes.ByteType | DataTypes.ShortType |
-              DataTypes.IntegerType | DataTypes.LongType | DataTypes.FloatType |
-              DataTypes.DoubleType | DataTypes.StringType =>
-            true
-          case DataTypes.DateType | DataTypes.TimestampType =>
-            // TODO implement these types with tests for formatting options and timezone
-            false
-          case _: MapType | _: ArrayType =>
-            // Spark supports map and array in StructsToJson but this is not yet
-            // implemented in Comet
-            false
-          case _ => false
-        }
-      }
-
       val isSupported = expr.child.dataType match {
         case s: StructType =>
           s.fields.forall(f => isSupportedType(f.dataType))
@@ -166,4 +146,87 @@ object CometStructsToJson extends CometExpressionSerde[StructsToJson] {
       }
     }
   }
+
+  def isSupportedType(dt: DataType): Boolean = {
+    dt match {
+      case StructType(fields) =>
+        fields.forall(f => isSupportedType(f.dataType))
+      case DataTypes.BooleanType | DataTypes.ByteType | DataTypes.ShortType |
+          DataTypes.IntegerType | DataTypes.LongType | DataTypes.FloatType |
+          DataTypes.DoubleType | DataTypes.StringType =>
+        true
+      case DataTypes.DateType | DataTypes.TimestampType =>
+        // TODO implement these types with tests for formatting options and timezone
+        false
+      case _: MapType | _: ArrayType =>
+        // Spark supports map and array in StructsToJson but this is not yet
+        // implemented in Comet
+        false
+      case _ => false
+    }
+  }
+}
+
+object CometJsonToStructs extends CometExpressionSerde[JsonToStructs] {
+
+  override def getSupportLevel(expr: JsonToStructs): SupportLevel = {
+    // this feature is partially implemented and not comprehensively tested yet
+    Incompatible()
+  }
+
+  override def convert(
+      expr: JsonToStructs,
+      inputs: Seq[Attribute],
+      binding: Boolean): Option[ExprOuterClass.Expr] = {
+
+    if (expr.schema == null) {
+      withInfo(expr, "from_json requires explicit schema")
+      return None
+    }
+
+    def isSupportedType(dt: DataType): Boolean = {
+      dt match {
+        case StructType(fields) =>
+          fields.nonEmpty && fields.forall(f => isSupportedType(f.dataType))
+        case DataTypes.IntegerType | DataTypes.LongType | DataTypes.FloatType |
+            DataTypes.DoubleType | DataTypes.BooleanType | DataTypes.StringType =>
+          true
+        case _ => false
+      }
+    }
+
+    val schemaType = expr.schema
+    if (!isSupportedType(schemaType)) {
+      withInfo(expr, "from_json: Unsupported schema type")
+      return None
+    }
+
+    val options = expr.options
+    if (options.nonEmpty) {
+      val mode = options.getOrElse("mode", "PERMISSIVE")
+      if (mode != "PERMISSIVE") {
+        withInfo(expr, s"from_json: Only PERMISSIVE mode supported, got: $mode")
+        return None
+      }
+      val knownOptions = Set("mode")
+      val unknownOpts = options.keySet -- knownOptions
+      if (unknownOpts.nonEmpty) {
+        withInfo(expr, s"from_json: Ignoring unsupported options: ${unknownOpts.mkString(", ")}")
+      }
+    }
+
+    // Convert child expression and schema to protobuf
+    for {
+      childProto <- exprToProtoInternal(expr.child, inputs, binding)
+      schemaProto <- serializeDataType(schemaType)
+    } yield {
+      val fromJson = ExprOuterClass.FromJson
+        .newBuilder()
+        .setChild(childProto)
+        .setSchema(schemaProto)
+        .setTimezone(expr.timeZoneId.getOrElse("UTC"))
+        .build()
+      ExprOuterClass.Expr.newBuilder().setFromJson(fromJson).build()
+    }
+  }
 }
