@@ -19,9 +19,12 @@
 
 package org.apache.comet.shims
 
-import org.apache.spark.sql.SQLQueryTestHelper
+import org.apache.spark.sql.{SparkSession, SQLQueryTestHelper}
 
 trait ShimCometTPCHQuerySuite extends SQLQueryTestHelper {
-  // no need to define getNormalizedQueryExecutionResult in this version of the shim because it exists
-  // in SQLQueryTestHelper in Spark 3.5
+  protected def getNormalizedQueryExecutionResult(
+      session: SparkSession,
+      sql: String): (String, Seq[String]) = {
+    getNormalizedResult(session, sql)
+  }
 }
