@@ -19,6 +19,4 @@
 
 package org.apache.spark.sql.comet.shims
 
-trait ShimCometTPCDSQuerySuite {
-
-}
+trait ShimCometTPCDSQuerySuite {}
